package generalidades;

import java.util.Iterator;
import java.util.List;

/**
 * Clase que contiene las distintas utilidades de carcter general que se usarn
 * para resolver el conjunto de problemas.
 * 
 * @author Jos Manuel Ponce
 */
public class Utilidades
{
	/**
	 * Mtodo que comprueba si los valores de la lista pasada como parmetro
	 * (que se corresponde con la lista de puntos o de etiquetas) son vlidos.
	 * Se consideran vlidos todos los valores mayores o iguales que cero.
	 * 
	 * @param lista
	 *            List<Integer> Lista de puntos o etiquetas
	 * @return boolean Cierto si todos los valores de la lista son vlidos
	 */
	public static boolean valoresValidos(List<Integer> lista)
	{
		boolean validos = true;
		Integer valor;
		Iterator<Integer> it = lista.listIterator();
		while (it.hasNext() && validos)
		{
			valor = it.next();
			if (0 >= valor)
			{
				validos = false;
			}
		}
		return validos;
	}
	
	/**
	 * Mtodo que comprueba si todos los puntos se encuentran ordenados
	 * ascendentemente. Se consideran que los puntos estn ordenados si
	 * cualquier punto (exceptuando el primero) es mayor o igual que el
	 * anterior.
	 * 
	 * @param puntos
	 *            List<Integer> Lista de puntos
	 * @return boolean Cierto si todos los puntos se encuentran ordenados
	 *         ascendentemente
	 */
	public static boolean puntosOrdenados(List<Integer> puntos)
	{
		boolean ordenados = true;
		Integer valorAnterior = -1;
		Integer valorActual;
		Iterator<Integer> it = puntos.listIterator();
		while (it.hasNext() && ordenados)
		{
			valorActual = it.next();
			if (valorActual < valorAnterior)
			{
				ordenados = false;
			} else
			{
				valorAnterior = valorActual;
			}
		}
		return ordenados;
	}
	
	/**
	 * Mtodo de ordenacin "burbuja". Ordena ascendentemente los elementos de
	 * la lista pasada como parmetro.
	 * 
	 * @param lista
	 *            List<Integer> Lista desordenada
	 * @return ListaEnteros Lista ordenada
	 */
	public static ListaEnteros ordenarListaInteger(
			List<Integer> lista)
	{
		int tam = lista.size();
		ListaEnteros dev = new ListaEnteros();
		
		int i, j;
		Integer aux;
		for (i = 0; i < tam; i++)
		{
			for (j = 0; j < tam - 1; j++)
			{
				if (lista.get(j) > lista.get(j + 1))
				{
					aux = lista.get(j);
					lista.set(j, lista.get(j + 1));
					lista.set(j + 1, aux);
				}
			}
		}
		
		dev.setLista(lista);
		return dev;
	}
	
	/**
	 * Mtodo de ordenacin "burbuja". Ordena ascendentemente los elementos de
	 * las listas pasadas como parmetro.
	 * 
	 * @param listaPuntos
	 *            List<Integer> Lista desordenada con los puntos
	 * @param listaEtiquetas
	 *            List<Integer> Lista desordenada con las etiquetas
	 * @return ConjuntoListas Listas de puntos y etiquetas ordenadas
	 */
	public static ConjuntoListas ordenarListasInteger(
			List<Integer> listaPuntos, List<Integer> listaEtiquetas)
	{
		int tam = listaPuntos.size();
		ConjuntoListas conjuntoDev;
		
		int i, j;
		Integer aux;
		for (i = 0; i < tam; i++)
		{
			for (j = 0; j < tam - 1; j++)
			{
				if (listaPuntos.get(j) > listaPuntos.get(j + 1))
				{
					aux = listaPuntos.get(j);
					listaPuntos.set(j, listaPuntos.get(j + 1));
					listaPuntos.set(j + 1, aux);
					aux = listaEtiquetas.get(j);
					listaEtiquetas.set(j, listaEtiquetas.get(j + 1));
					listaEtiquetas.set(j + 1, aux);
				}
			}
		}
		conjuntoDev = new ConjuntoListas(listaPuntos, listaEtiquetas);
		return conjuntoDev;
	}
	
	/**
	 * Mtodo de ordenacin "burbuja". Ordena ascendentemente los elementos de
	 * las listas pasadas como parmetro.
	 * 
	 * @param listaPuntos
	 *            List<Integer> Lista desordenada con los puntos
	 * @param listaAlturas
	 *            List<Integer> Lista desordenada con las alturas de las etiquetas
	 * @param listaAnchuras
	 *            List<Integer> Lista desordenada con las anchuras de las etiquetas
	 * @return ConjuntoTresListas Listas de puntos, anchuras y alturas de etiquetas ordenadas
	 */
	public static ConjuntoTresListas ordenarTresListasInteger(
			List<Integer> listaPuntos, List<Integer> listaAlturas, List<Integer> listaAnchuras)
	{
		int tam = listaPuntos.size();
		ConjuntoTresListas conjuntoDev;
		
		int i, j;
		Integer aux;
		for (i = 0; i < tam; i++)
		{
			for (j = 0; j < tam - 1; j++)
			{
				if (listaPuntos.get(j) > listaPuntos.get(j + 1))
				{
					aux = listaPuntos.get(j);
					listaPuntos.set(j, listaPuntos.get(j + 1));
					listaPuntos.set(j + 1, aux);
					aux = listaAlturas.get(j);
					listaAlturas.set(j, listaAlturas.get(j + 1));
					listaAlturas.set(j + 1, aux);
					aux = listaAnchuras.get(j);
					listaAnchuras.set(j, listaAnchuras.get(j + 1));
					listaAnchuras.set(j + 1, aux);
				}
			}
		}
		conjuntoDev = new ConjuntoTresListas(listaPuntos, listaAlturas, listaAnchuras);
		return conjuntoDev;
	}
	
	/**
	 * Mtodo que se encarga de imprimir una lista de Integer por pantalla.
	 * 
	 * @param lista
	 *            List<Integer> Lista que se imprime
	 */
	public static void imprimeListaInteger(List<Integer> lista)
	{
		String cadenaLista = "( ";
		int numElementos = lista.size();
		for (int i = 0; i < numElementos; i++)
		{
			cadenaLista += lista.get(i);
			if ((numElementos - 1) != i)
			{
				cadenaLista += " , ";
			}
		}
		cadenaLista += " )";
		System.out.println(cadenaLista);
	}
	
	/**
	 * Mtodo que se encarga de imprimir una lista de String por pantalla.
	 * 
	 * @param lista
	 *            List<String> Lista que se imprime
	 */
	public static void imprimeListaString(List<String> lista)
	{
		String cadenaLista = "( ";
		int numElementos = lista.size();
		for (int i = 0; i < numElementos; i++)
		{
			cadenaLista += lista.get(i);
			if ((numElementos - 1) != i)
			{
				cadenaLista += " , ";
			}
		}
		cadenaLista += " )";
		System.out.println(cadenaLista);
	}
	
	/**
	 * Mtodo que devuelve el tamao del elemento Integer mayor de la lista que
	 * se le pasa como parmetro.
	 * 
	 * @param lista
	 *            List<Integer> Lista de elementos Integer
	 * @return int Tamao del elemento mayor de la lista
	 */
	public static int maxValorLista(List<Integer> lista)
	{
		int maxValor = -1;
		Iterator<Integer> it = lista.listIterator();
		Integer valor;
		while (it.hasNext())
		{
			valor = it.next();
			if (valor > maxValor)
			{
				maxValor = valor;
			}
		}
		
		return maxValor;
	}
	
	/**
	 * Mtodo que devuelve el mdulo 10 del entero que se le pasa como
	 * parmetro.
	 * 
	 * @param num
	 *            Entero del que se quiere hallar el mdulo 10
	 * @return Mdulo 10 del entero que se le pasa como parmetro
	 */
	public static int mod10(int num)
	{
		int dev = num % 10;
		return dev;
	}
	
}
