package lineaHorizontal.h_4p;

import generalidades.Constantes;
import generalidades.Utilidades;

import java.util.ArrayList;
import java.util.List;

import generalidades.ConjuntoListas;
import lineaHorizontal.SombraHorizontal;
import lineaHorizontal.UtilidadesHorizontal;

/**
 * Clase que contiene los mtodos necesarios para solucionar el problema H-4P.
 * 
 * @author Jos Manuel Ponce
 */
public class Solucion_H_4P
{
	/**
	 * Lista de puntos.
	 */
	private List<Integer> puntos;
	
	/**
	 * Lista de etiquetas.
	 */
	private List<Integer> etiquetas;
	
	/**
	 * Lista de posiciones finales de las etiquetas respecto a los puntos.
	 */
	private List<String> resultado;
	
	/**
	 * Lista auxiliar cuyos elementos denotan si estn ocupadas por una etiqueta
	 * las posiciones que representan por encima de la lnea de entrada.
	 */
	private List<Integer> lineaSuperior;
	
	/**
	 * Lista auxiliar cuyos elementos denotan si estn ocupadas por una etiqueta
	 * las posiciones que representan por debajo de la lnea de entrada.
	 */
	private List<Integer> lineaInferior;
	
	/**
	 * Nmero de pares punto-etiqueta.
	 */
	private int numElementos;
	
	/**
	 * Nmero de divisiones de la lnea de entrada.
	 */
	private int longitudLinea;
	
	/**
	 * Objeto SombraHorizontal.
	 */
	private SombraHorizontal sombra;
	
	/**
	 * Constructor sin parmetros.
	 */
	public Solucion_H_4P()
	{
		super();
	}
	
	/**
	 * Constructor con parmetros.
	 * 
	 * @param puntos
	 *            List<Integer> Lista de puntos
	 * @param etiquetas
	 *            List<Integer> Lista de etiquetas
	 */
	public Solucion_H_4P(List<Integer> puntos, List<Integer> etiquetas)
	{
		this.puntos = puntos;
		this.etiquetas = etiquetas;
		this.numElementos = puntos.size();
		this.resultado = new ArrayList<String>();
		this.lineaSuperior = new ArrayList<Integer>();
		this.lineaInferior = new ArrayList<Integer>();
	}
	
	/**
	 * Mtodo que imprime la solucin al problema, si la hubiese.
	 */
	public void obtenerSolucion()
	{
		ConjuntoListas conjunto;
		SombraHorizontal sombraTL;
		SombraHorizontal sombraBL;
		SombraHorizontal sombraTR;
		SombraHorizontal sombraBR;
		boolean haySolucion = true;
		
		if ((Utilidades.valoresValidos(this.puntos))
				&& (Utilidades.valoresValidos(this.etiquetas)))
		{
			if (!Utilidades.puntosOrdenados(puntos))
			{
				conjunto = Utilidades.ordenarListasInteger(this.puntos,
						this.etiquetas);
				this.puntos = conjunto.getListaPuntos();
				this.etiquetas = conjunto.getListaEtiquetas();
				System.out
						.println("La lista de puntos no est ordenada. Procedemos a ordenarla.");
				System.out
						.println("Los pares (punto , etiqueta) indicados han sido:");
				for (int i = 0; i < numElementos; i++)
				{
					System.out.print("par " + (i + 1) + ": ("
							+ this.puntos.get(i) + " , "
							+ this.etiquetas.get(i) + ")\n");
				}
			} else
			{
				System.out
						.println("La lista de puntos est ordenada. Procedemos a la resolucin del problema:");
			}
			inicializaValores();
			
			for (int i = 0; (i < this.numElementos) && haySolucion; i++)
			{
				sombraTL = this.probarEtiquetaTL(this.puntos.get(i),
						this.etiquetas.get(i));
				sombraBL = this.probarEtiquetaBL(this.puntos.get(i),
						this.etiquetas.get(i));
				sombraTR = this.probarEtiquetaTR(this.puntos.get(i),
						this.etiquetas.get(i));
				sombraBR = this.probarEtiquetaBR(this.puntos.get(i),
						this.etiquetas.get(i));
				System.out.println(">> Sombras resultantes para el punto "
						+ (i + 1) + " y anteriores:");
				int idSombraMenor = UtilidadesHorizontal.hallarSombraMenor4(
						sombraTL, sombraBL, sombraTR, sombraBR);
				if (null == sombraTL)
				{
					System.out.println("Sombra TL: (-- , --)");
				} else
				{
					System.out.println("Sombra TL: (" + (sombraTL.getTk())
							+ " , " + (sombraTL.getBk()) + ")");
				}
				if (null == sombraBL)
				{
					System.out.println("Sombra BL: (-- , --)");
				} else
				{
					System.out.println("Sombra BL: (" + (sombraBL.getTk())
							+ " , " + (sombraBL.getBk()) + ")");
				}
				if (null == sombraTR)
				{
					System.out.println("Sombra TR: (-- , --)");
				} else
				{
					System.out.println("Sombra TR: (" + (sombraTR.getTk())
							+ " , " + (sombraTR.getBk()) + ")");
				}
				if (null == sombraBR)
				{
					System.out.println("Sombra BR: (-- , --)");
				} else
				{
					System.out.println("Sombra BR: (" + (sombraBR.getTk())
							+ " , " + (sombraBR.getBk()) + ")");
				}
				if (-1 != idSombraMenor)
				{
					if (1 == idSombraMenor)
					{
						this.aplicarEtiquetaTL(this.puntos.get(i),
								this.etiquetas.get(i), i + 1);
						this.resultado.set(i, Constantes.POS_ARRIBA_IZQUIERDA);
					} else if (2 == idSombraMenor)
					{
						this.aplicarEtiquetaBL(this.puntos.get(i),
								this.etiquetas.get(i), i + 1);
						this.resultado.set(i, Constantes.POS_ABAJO_IZQUIERDA);
					} else if (3 == idSombraMenor)
					{
						this.aplicarEtiquetaTR(this.puntos.get(i),
								this.etiquetas.get(i), i + 1);
						this.resultado.set(i, Constantes.POS_ARRIBA_DERECHA);
					} else if (4 == idSombraMenor)
					{
						this.aplicarEtiquetaBR(this.puntos.get(i),
								this.etiquetas.get(i), i + 1);
						this.resultado.set(i, Constantes.POS_ABAJO_DERECHA);
					}
				} else
				{
					haySolucion = false;
				}
			}
			
			System.out.println("Se han obtenido las siguientes posiciones:");
			Utilidades.imprimeListaString(resultado);
			if (!haySolucion)
			{
				System.out
						.println("No se ha podido encontrar una solucin vlida a ");
				System.out
						.println("este problema con los datos que ha proporcionado.");
				System.out
						.println("Grficamente, la solucin parcial obtenida ha sido la siguiente:");
			}
			if (haySolucion)
			{
				System.out
						.println("Grficamente, la solucin del problema se puede representar como sigue:");
			}
			UtilidadesHorizontal.representar(this.longitudLinea, this.puntos,
					this.lineaSuperior, this.lineaInferior);
			System.out.println(">> Fin del problema.");
		} else
		{
			System.out
					.println("Los valores de los puntos y etiquetas han de ser mayores que cero.");
			System.out.println("Vuelva a ejecutar el programa si lo desea.");
		}
	}
	
	/**
	 * Mtodo que inicializa las listas "resultado", "lineaSuperior" y
	 * "lineaInferior".
	 */
	public void inicializaValores()
	{
		this.sombra = new SombraHorizontal(0, 0);
		this.longitudLinea = this.puntos.get(numElementos - 1) + 1 + Utilidades.maxValorLista(this.etiquetas);
		for (int i = 0; i <= this.longitudLinea; i++)
		{
			this.lineaSuperior.add(i, null);
			this.lineaInferior.add(i, null);
		}
		for (int i = 0; i < this.numElementos; i++)
		{
			this.resultado.add(i, "--");
		}
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin TL para comprobar el tamao de la sombra
	 * resultante.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return SombraHorizontal Sombra virtual, null si no se puede
	 */
	public SombraHorizontal probarEtiquetaTL(Integer punto, Integer etiqueta)
	{
		SombraHorizontal dev = new SombraHorizontal();
		boolean sigue = true;
		int i;
		for (i = punto - 1; (i >= 0) && (i >= punto - etiqueta) && sigue; i--)
		{
			if (((i <= sombra.getTk()) && (0 < sombra.getTk()))
					|| (null != this.lineaSuperior.get(i)))
			{
				sigue = false;
			}
		}
		if (sigue)
		{
			dev.setTk(punto - 1);
			dev.setBk(this.sombra.getBk());
		} else
		{
			dev = null;
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin BL para comprobar el tamao de la sombra
	 * resultante.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return SombraHorizontal Sombra virtual, null si no se puede
	 */
	public SombraHorizontal probarEtiquetaBL(Integer punto, Integer etiqueta)
	{
		SombraHorizontal dev = new SombraHorizontal();
		boolean sigue = true;
		int i;
		for (i = punto - 1; (i >= 0) && (i >= punto - etiqueta) && sigue; i--)
		{
			if (((i <= sombra.getBk()) && (0 < sombra.getBk()))
					|| (null != this.lineaInferior.get(i)))
			{
				sigue = false;
			}
		}
		if (sigue)
		{
			dev.setTk(this.sombra.getTk());
			dev.setBk(punto - 1);
		} else
		{
			dev = null;
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin TR para comprobar el tamao de la sombra
	 * resultante.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return SombraHorizontal Sombra virtual, null si no se puede
	 */
	public SombraHorizontal probarEtiquetaTR(Integer punto, Integer etiqueta)
	{
		SombraHorizontal dev = new SombraHorizontal(-1, -1);
		boolean sigue = true;
		int i;
		for (i = punto + 1; (i <= this.longitudLinea)
				&& (i <= punto + etiqueta) && sigue; i++)
		{
			if (((i <= sombra.getTk()) && (0 < sombra.getTk()))
					|| (null != this.lineaSuperior.get(i)))
			{
				sigue = false;
			}
		}
		if (sigue)
		{
			dev.setTk(punto + etiqueta);
			dev.setBk(this.sombra.getBk());
		} else
		{
			dev = null;
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin BR para comprobar el tamao de la sombra
	 * resultante.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @return SombraHorizontal Sombra virtual, null si no se puede
	 */
	public SombraHorizontal probarEtiquetaBR(Integer punto, Integer etiqueta)
	{
		SombraHorizontal dev = new SombraHorizontal(-1, -1);
		boolean sigue = true;
		int i;
		for (i = punto + 1; (i <= this.longitudLinea)
				&& (i <= punto + etiqueta) && sigue; i++)
		{
			if (((i <= sombra.getBk()) && (0 < sombra.getBk()))
					|| (null != this.lineaInferior.get(i)))
			{
				sigue = false;
			}
		}
		if (sigue)
		{
			dev.setTk(this.sombra.getTk());
			dev.setBk(punto + etiqueta);
		} else
		{
			dev = null;
		}
		return dev;
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @param indice
	 *            ndice aplicado a la etiqueta que se aade
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaTL(Integer punto, Integer etiqueta, int indice)
	{
		indice = indice % 10;
		boolean dev = false;
		int i;
		for (i = punto - 1; (i >= 0) && (i >= punto - etiqueta); i--)
		{
			this.lineaSuperior.set(i, indice);
		}
		this.sombra.setTk(punto - 1);
		this.sombra.setBk(this.sombra.getBk());
		dev = true;
		return dev;
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin BL.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @param indice
	 *            ndice aplicado a la etiqueta que se aade
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaBL(Integer punto, Integer etiqueta, int indice)
	{
		indice = indice % 10;
		boolean dev = false;
		int i;
		for (i = punto - 1; (i >= 0) && (i >= punto - etiqueta); i--)
		{
			this.lineaInferior.set(i, indice);
		}
		this.sombra.setTk(this.sombra.getTk());
		this.sombra.setBk(punto - 1);
		dev = true;
		return dev;
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin TR.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @param indice
	 *            ndice aplicado a la etiqueta que se aade
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaTR(Integer punto, Integer etiqueta, int indice)
	{
		indice = indice % 10;
		boolean dev = false;
		int i;
		for (i = punto + 1; (i <= this.longitudLinea)
				&& (i <= punto + etiqueta); i++)
		{
			this.lineaSuperior.set(i, indice);
		}
		this.sombra.setTk(punto + etiqueta);
		this.sombra.setBk(this.sombra.getBk());
		dev = true;
		return dev;
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin BR.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param etiqueta
	 *            Tamao de la etiqueta que se va a aadir
	 * @param indice
	 *            ndice aplicado a la etiqueta que se aade
	 * @return boolean Cierto si se aade correctamente la etiqueta
	 */
	public boolean aplicarEtiquetaBR(Integer punto, Integer etiqueta, int indice)
	{
		indice = indice % 10;
		boolean dev = false;
		int i;
		for (i = punto + 1; (i <= this.longitudLinea)
				&& (i <= punto + etiqueta); i++)
		{
			this.lineaInferior.set(i, indice);
		}
		this.sombra.setTk(this.sombra.getTk());
		this.sombra.setBk(punto + etiqueta);
		dev = true;
		return dev;
	}
}
