package lineaOblicua.orm_4p;

import generalidades.ConjuntoTresListas;
import generalidades.Constantes;
import generalidades.Utilidades;

import java.util.ArrayList;
import java.util.List;

import lineaOblicua.UtilidadesOblicua;

/**
 * Clase que contiene los mtodos necesarios para solucionar el problema OC-4P.
 * 
 * @author Jos Manuel Ponce
 */
public class Solucion_ORM_4P
{
	/**
	 * Lista de puntos.
	 */
	private List<Integer> puntos;
	
	/**
	 * Lista de posiciones finales de las etiquetas respecto a los puntos.
	 */
	private List<String> resultado;
	
	/**
	 * Nmero de pares punto-etiqueta.
	 */
	private int numElementos;
	
	/**
	 * Nmero de divisiones de la lnea de entrada.
	 */
	private int longitudLinea;
	
	/**
	 * Anchuras de las etiquetas.
	 */
	private List<Integer> anchuras;
	
	/**
	 * Alturas de las etiquetas.
	 */
	private List<Integer> alturas;
	
	/**
	 * Objeto que almacena la posicin de las etiquetas que ya se han aadido.
	 */
	private boolean[][] espacioEtiquetas;
	
	/**
	 * Constructor sin parmetros.
	 */
	public Solucion_ORM_4P()
	{
		super();
	}
	
	/**
	 * Constructor con parmetros.
	 * 
	 * @param puntos
	 *            List<Integer> Lista de puntos
	 * @param alturaEtiquetas
	 *            List<Integer> Altura de las etiquetas
	 * @param anchuraEtiquetas
	 *            List<Integer> Anchura de las etiquetas
	 */
	public Solucion_ORM_4P(List<Integer> puntos, List<Integer> alturas,
			List<Integer> anchuras)
	{
		this.puntos = puntos;
		this.numElementos = puntos.size();
		this.resultado = new ArrayList<String>();
		this.anchuras = anchuras;
		this.alturas = alturas;
	}
	
	/**
	 * Mtodo que imprime la solucin al problema, si la hubiese.
	 */
	public void obtenerSolucion()
	{
		boolean haySolucion = true;
		
		if (Utilidades.valoresValidos(this.puntos)
				&& Utilidades.valoresValidos(this.alturas)
				&& Utilidades.valoresValidos(this.anchuras))
		{
			if (!Utilidades.puntosOrdenados(puntos))
			{
				ConjuntoTresListas conjunto = Utilidades
						.ordenarTresListasInteger(this.puntos, this.alturas,
								this.anchuras);
				this.puntos = conjunto.getListaPuntos();
				this.alturas = conjunto.getAlturas();
				this.anchuras = conjunto.getAnchuras();
				System.out
						.println("La lista de puntos no est ordenada. Procedemos a ordenarla.");
				System.out
						.println("Las tripletas (punto, altura, anchura) indicadas han sido:");
				for (int i = 0; i < this.numElementos; i++)
				{
					System.out.println("( " + this.puntos.get(i) + " , "
							+ this.alturas.get(i) + " , "
							+ this.anchuras.get(i) + " )");
				}
			} else
			{
				System.out
						.println("La lista de puntos est ordenada. Procedemos a la resolucin del problema:");
			}
			inicializaValores();
			
			for (int i = 0; (i < this.numElementos) && haySolucion; i++)
			{
				if (this.probarEtiquetaBL(this.puntos.get(i), this.alturas.get(i), this.anchuras.get(i)))
				{
					this.aplicarEtiquetaBL(this.puntos.get(i), this.alturas.get(i), this.anchuras.get(i));
					this.resultado.set(i, Constantes.POS_ABAJO_IZQUIERDA);
					System.out
							.println("\nEstado del espacio despus de etiquetar el punto "
									+ (this.puntos.get(i)));
					UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
				} else if (this.probarEtiquetaTL(this.puntos.get(i), this.alturas.get(i), this.anchuras.get(i)))
				{
					this.aplicarEtiquetaTL(this.puntos.get(i), this.alturas.get(i), this.anchuras.get(i));
					this.resultado.set(i, Constantes.POS_ARRIBA_IZQUIERDA);
					System.out
							.println("\nEstado del espacio despus de etiquetar el punto "
									+ (this.puntos.get(i)));
					UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
				} else if (this.probarEtiquetaBR(this.puntos.get(i), this.alturas.get(i), this.anchuras.get(i)))
				{
					this.aplicarEtiquetaBR(this.puntos.get(i), this.alturas.get(i), this.anchuras.get(i));
					this.resultado.set(i, Constantes.POS_ABAJO_DERECHA);
					System.out
							.println("\nEstado del espacio despus de etiquetar el punto "
									+ (this.puntos.get(i)));
					UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
				} else if (this.probarEtiquetaTR(this.puntos.get(i), this.alturas.get(i), this.anchuras.get(i)))
				{
					this.aplicarEtiquetaTR(this.puntos.get(i), this.alturas.get(i), this.anchuras.get(i));
					this.resultado.set(i, Constantes.POS_ARRIBA_DERECHA);
					System.out
							.println("\nEstado del espacio despus de etiquetar el punto "
									+ (this.puntos.get(i)));
					UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
				} else
				{
					haySolucion = false;
				}
			}
			
			System.out.println("\nSe han obtenido las siguientes posiciones:");
			Utilidades.imprimeListaString(resultado);
			if (!haySolucion)
			{
				System.out
						.println("No se ha podido encontrar una solucin vlida a ");
				System.out
						.println("este problema con los datos que ha proporcionado.");
				System.out
						.println("Grficamente, la solucin parcial obtenida ha sido la siguiente:");
			}
			if (haySolucion)
			{
				System.out
						.println("\nGrficamente, la solucin del problema se puede representar como sigue:");
			}
			UtilidadesOblicua.representarEspacio(this.espacioEtiquetas);
			System.out.println(">> Fin del problema.");
		} else
		{
			System.out
					.println("Los valores de los puntos y etiquetas han de ser mayores que cero.");
			System.out.println("Vuelva a ejecutar el programa si lo desea.");
		}
	}
	
	/**
	 * Mtodo que inicializa las listas "resultado", "lineaSuperior" y
	 * "lineaInferior".
	 */
	public void inicializaValores()
	{
		this.longitudLinea = this.puntos.get(numElementos - 1) + 2 + Utilidades.maxValorLista(this.anchuras);
		this.espacioEtiquetas = new boolean[this.longitudLinea][this.longitudLinea];
		for (int i = 0; i < this.longitudLinea; i++)
		{
			for (int j = 0; j < this.longitudLinea; j++)
			{
				this.espacioEtiquetas[i][j] = false;
			}
		}
		for (int i = 0; i < this.numElementos; i++)
		{
			this.resultado.add(i, "--");
		}
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin TL para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param alturaE
	 *            Altura de la etiqueta
	 * @param anchuraE
	 *            Anchura de la etiqueta
	 * @return boolean Cierto si se puede colocar la etiqueta en esta posicin
	 */
	public boolean probarEtiquetaTL(Integer punto, Integer alturaE, Integer anchuraE)
	{
		boolean dev = true;
		int x;
		int y = punto;
		for (int desp = 0; (desp < alturaE)
				&& (y + desp < this.longitudLinea) && dev; desp++)
		{
			for (x = punto - 1; (x >= 0) && (x >= punto - anchuraE)
					&& dev; x--)
			{
				if (this.espacioEtiquetas[y + desp][x])
				{
					dev = false;
				}
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin BL para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param alturaE
	 *            Altura de la etiqueta
	 * @param anchuraE
	 *            Anchura de la etiqueta
	 * @return boolean Cierto si se puede colocar la etiqueta en esta posicin
	 */
	public boolean probarEtiquetaBL(Integer punto, Integer alturaE, Integer anchuraE)
	{
		boolean dev = true;
		int x;
		int y = punto - 1;
		for (int desp = 0; (desp < alturaE) && (y - desp >= 0)
				&& dev; desp++)
		{
			for (x = punto - 1; (x >= 0) && (x >= punto - anchuraE)
					&& dev; x--)
			{
				if (this.espacioEtiquetas[y - desp][x])
				{
					dev = false;
				}
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin TR para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param alturaE
	 *            Altura de la etiqueta
	 * @param anchuraE
	 *            Anchura de la etiqueta
	 * @return boolean Cierto si se puede colocar la etiqueta en esta posicin
	 */
	public boolean probarEtiquetaTR(Integer punto, Integer alturaE, Integer anchuraE)
	{
		boolean dev = true;
		int x;
		int y = punto;
		for (int desp = 0; (desp < alturaE)
				&& (y + desp < this.longitudLinea) && dev; desp++)
		{
			for (x = punto; (x < this.longitudLinea)
					&& (x < punto + anchuraE) && dev; x++)
			{
				if (this.espacioEtiquetas[y + desp][x])
				{
					dev = false;
				}
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que simula el agregar una nueva etiqueta en el punto indicado de
	 * la lnea en la posicin BR para comprobar el tamao de la sombra
	 * resultante y, en su caso, agrega la etiqueta y pasa al siguiente punto.
	 * 
	* @param punto
	 *            Punto a etiquetar
	 * @param alturaE
	 *            Altura de la etiqueta
	 * @param anchuraE
	 *            Anchura de la etiqueta
	 * @return boolean Cierto si se puede colocar la etiqueta en esta posicin
	 */
	public boolean probarEtiquetaBR(Integer punto, Integer alturaE, Integer anchuraE)
	{
		boolean dev = true;
		int x;
		int y = punto - 1;
		for (int desp = 0; (desp < alturaE) && (y - desp >= 0)
				&& dev; desp++)
		{
			for (x = punto; (x < this.longitudLinea)
					&& (x < punto + anchuraE) && dev; x++)
			{
				if (this.espacioEtiquetas[y - desp][x])
				{
					dev = false;
				}
			}
		}
		return dev;
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin TL.
	 * 
	* @param punto
	 *            Punto a etiquetar
	 * @param alturaE
	 *            Altura de la etiqueta
	 * @param anchuraE
	 *            Anchura de la etiqueta
	 */
	public void aplicarEtiquetaTL(Integer punto, Integer alturaE, Integer anchuraE)
	{
		int x;
		int y = punto;
		for (int desp = 0; (desp < alturaE)
				&& (y + desp < this.longitudLinea); desp++)
		{
			for (x = punto - 1; (x >= 0) && (x >= punto - anchuraE); x--)
			{
				this.espacioEtiquetas[y + desp][x] = true;
			}
		}
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin BL.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param alturaE
	 *            Altura de la etiqueta
	 * @param anchuraE
	 *            Anchura de la etiqueta
	 */
	public void aplicarEtiquetaBL(Integer punto, Integer alturaE, Integer anchuraE)
	{
		int x;
		int y = punto - 1;
		for (int desp = 0; (desp < alturaE) && (y - desp >= 0); desp++)
		{
			for (x = punto - 1; (x >= 0) && (x >= punto - anchuraE); x--)
			{
				this.espacioEtiquetas[y - desp][x] = true;
			}
		}
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin TR.
	 * 
	 * @param punto
	 *            Punto a etiquetar
	 * @param alturaE
	 *            Altura de la etiqueta
	 * @param anchuraE
	 *            Anchura de la etiqueta
	 */
	public void aplicarEtiquetaTR(Integer punto, Integer alturaE, Integer anchuraE)
	{
		int x;
		int y = punto;
		for (int desp = 0; (desp < alturaE)
				&& (y + desp < this.longitudLinea); desp++)
		{
			for (x = punto; (x < this.longitudLinea)
					&& (x < punto + anchuraE); x++)
			{
				this.espacioEtiquetas[y + desp][x] = true;
			}
		}
	}
	
	/**
	 * Mtodo que agrega una nueva etiqueta en el punto indicado de la lnea en
	 * la posicin BR.
	 * 
	* @param punto
	 *            Punto a etiquetar
	 * @param alturaE
	 *            Altura de la etiqueta
	 * @param anchuraE
	 *            Anchura de la etiqueta
	 */
	public void aplicarEtiquetaBR(Integer punto, Integer alturaE, Integer anchuraE)
	{
		int x;
		int y = punto - 1;
		for (int desp = 0; (desp < alturaE) && (y - desp >= 0); desp++)
		{
			for (x = punto; (x < this.longitudLinea)
					&& (x < punto + anchuraE); x++)
			{
				this.espacioEtiquetas[y - desp][x] = true;
			}
		}
	}
	
}
